try:
    import re
    from behave import when
    import utilities
    from xml.dom import minidom
    utilities.safe_selenium_imports(__file__, globals())
except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.common.exceptions import NoSuchElementException, TimeoutException
# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.support.ui import WebDriverWait


@given('I have "{cert_type}" cert for "{vendor}" "{product_name}"')
def step_impl(context,cert_type,vendor, product_name):
    cert_version, platform, make_name = context.data['rhel'], context.data['arch'], context.data['make']
    context.execute_steps(u"""
                          Given I am logged into RHCERT
                          When  I click on the New Certification button
                          When  I select product "Red Hat Enterprise Linux"
                          And   I select RHEL version "%s" for "%s"
                          And   I select platform "%s"
                          And   I click on Next button
                          And   I select vendor "%s"
                          And   I select Make "%s"
                          And   I select Name "%s"
                          And   I click on Next button
                          Then  I see the "%s" certification created on rhcert
                          """ % (cert_version, cert_type, platform, vendor, make_name, product_name, cert_type))


@Then('I check "{cert_type}" sut info and save it')
def step_impl(context,cert_type):
    """In this step  systemc will got to configuration tab and from page_source its getting SUT info"""
    SUT = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    sut_info_link=context.browser.find_element_by_link_text(SUT).click()
    sut_info_source=context.browser.page_source
    xmldoc  = minidom.parseString(sut_info_source)
    context.data['rhel']=xmldoc.getElementsByTagName('release')[0].firstChild.nodeValue
    context.data['arch']=xmldoc.getElementsByTagName('arch')[0].firstChild.nodeValue
    context.data['make']=xmldoc.getElementsByTagName('make')[0].firstChild.nodeValue
@when("I select tests for cloud cert")
def step_impl(context):
    cloud_configuration = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/configuration']")
    cloud_configuration.click()
    cloud_security = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/security']")
    cloud_security.click()


@when("I click on the Run Selected Button")
def step_impl(context):
    result_type = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Run Selected']")
    result_type.click()

@when('I select RHEL version of "{cert_type}" SUT')
def step_impl(context, cert_type):
    rhel_version_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('rh_product_version_id'))
    rhel_version_option_link.select_by_visible_text(context.data['rhel'])


@when('I select platform of "{cert_type}" SUT')
def step_impl(context, cert_type):
    platform_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('architecture'))
    platform_option_link.select_by_visible_text(context.data['arch'])


@when("I give reason for supportable test checks")
def step_impl(context):
    try:
        webdriver.support.ui.WebDriverWait(context.browser, 1200).until(
            webdriver.support.expected_conditions.visibility_of_element_located((
                webdriver.common.by.By.XPATH, "//label[@for='prompt-answer']")))
        Check1 = context.browser.find_element_by_xpath("//label[@for='prompt-answer']").text
        check_package = Check1[-42:].strip()
        if check_package == ("Please describe the need for these packages:"):
            reason = context.browser.find_element_by_xpath("//input[@id='prompt-answer']")
            reason.send_keys("automation check")
            submit = context.browser.find_element_by_css_selector(
                "input[type='submit'][value='Submit']")
            submit.click()
            webdriver.support.ui.WebDriverWait(context.browser, 1200).until(
                webdriver.support.expected_conditions.visibility_of_element_located((
                    webdriver.common.by.By.XPATH, "//label[@for='prompt-answer']")))
        Check2 = context.browser.find_element_by_xpath("//label[@for='prompt-answer']").text
        check_repo = Check2[-42:].strip()
        if check_repo == ("Please describe the need for these repos:"):
            reason = context.browser.find_element_by_xpath("//input[@id='prompt-answer']")
            reason.send_keys("automation check")
            submit = context.browser.find_element_by_css_selector(
                "input[type='submit'][value='Submit']")
            submit.click()
    except common.exceptions.TimeoutException:
        pass

@when("I give reason for configuration test checks")
def step_impl(context):
    try:
        webdriver.support.ui.WebDriverWait(context.browser, 2500).until(
            webdriver.support.expected_conditions.visibility_of_element_located((
                webdriver.common.by.By.XPATH, "//label[@for='prompt-answer']")))
        Flag = True
        while Flag:
            Pattern = "Please supply an explanation why this service is required:"
            try:
                if context.browser.find_element_by_xpath(
                        "//label[@for='prompt-answer']").is_displayed():
                    Port = context.browser.find_element_by_xpath("//label[@for='prompt-answer']").text
                    check_port = Port[-59:].strip()
                    if re.match(Pattern, check_port):
                        reason = context.browser.find_element_by_css_selector(
                            "input[type='text'][name='prompt-answer']")
                        reason.send_keys("automation check")
                        submit = context.browser.find_element_by_css_selector(
                            "input[type='submit'][value='Submit']")
                        submit.click()
            except common.exceptions.NoSuchElementException:
                Flag = False
    except (common.exceptions.TimeoutException, common.exceptions.StaleElementReferenceException) as e:
        pass

@when("I select supportable test")
def step_impl(context):
    cloud_supportable = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/supportable']")
    cloud_supportable.click()


@when("I select configuration test")
def step_impl(context):
    cloud_configuration = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/configuration']")
    cloud_configuration.click()


@when("I select security test")
def step_impl(context):
    cloud_security = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/security']")
    cloud_security.click()


@when("I select supportable and configuration test")
def step_impl(context):
    cloud_supportable = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/supportable']")
    cloud_supportable.click()
    cloud_configuration = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/configuration']")
    cloud_configuration.click()


@when("I select supportable and security test")
def step_impl(context):
    cloud_supportable = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/supportable']")
    cloud_supportable.click()
    cloud_security = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/security']")
    cloud_security.click()


@when("I select configuration and security test")
def step_impl(context):
    cloud_configuration = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/configuration']")
    cloud_configuration.click()
    cloud_security = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='cloud/security']")
    cloud_security.click()
