try:
    from behave import given, when, then
    from rhcert import get_cert_id, get_rhcert_cert_id
    import utilities
    utilities.safe_selenium_imports(__file__, globals())

except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.support.ui import WebDriverWait

@given('I am logged in to CWE')
def step_impl(context):
    context.execute_steps(u"""
                          given I am on CWE login page
                          when I fill in the cwe login credentials
                          and I click on Submit button
                          """)


@when('I visit the CWE certification page for "{cert_type}"')
def step_impl(context, cert_type):
    context.browser.get(context.data['common']['cwe']['uri'] +
                        '/show.cgi?cert_nid=' +
                        str(get_cert_id(context, cert_type)))


@when('I visit the CWE certification page for "{cert_type}" certification created from rhcert')
def step_impl(context, cert_type):
    context.browser.get(context.data['common']['cwe']['uri'] +
                        '/show.cgi?cert_nid=' +
                        str(get_rhcert_cert_id(context, cert_type)))


@when("I click on the 'Dialog' tab")
def step_impl(context):
    context.browser.find_element_by_css_selector(
        "a[href='#dialog'][role='tab']").click()


@then('I should see a comment saying probe data passed to Caas')
def step_impl(context):
    expected_text = ['Received new test log',
                     'forwarded to CAAS successful']

    for text in expected_text:
        assert text in context.browser.page_source


@then('Validation reports should be attached to the Certification Page')
def step_impl(context):
    expected_text = ['validation report:result.json from CAAS',
                     'validation report:result.html from CAAS',
                     'validation report:validation_report.json from CAAS']
    for text in expected_text:
        assert text in context.browser.page_source


@when('I click on the create link')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 200).until(
        webdriver.support.expected_conditions.presence_of_element_located((webdriver.common.by.By.XPATH, '//*[@title="Create New Hardware Entry"]')))
    context.browser.find_element_by_xpath(
        '//*[@title="Create New Hardware Entry"]').click()


@then('I should see different certifications as options')
def step_impl(context):
    heading = context.browser.find_element_by_class_name('section-heading')
    assert heading.get_attribute('innerHTML') == 'Select Certification Type'


@then('I should see the login page')
def step_impl(context):
    assert context.browser.title == "Log in to Red Hat IDP"


@then('A "{cert_type}" certification should be created')
def step_impl(context, cert_type):
    cert_dict = {'cinder': 'OpenStack',
                 'neutron': 'OpenStack',
                 'sahara': 'OpenStack',
                 'cloud': 'Cloud',
                 'management': 'OpenStack',
                 'container': 'Container'}
    context.data[context.data['parents'][cert_type]][cert_type]['cert_id'] = \
        context.browser.find_element_by_css_selector("div.col-sm-8.div_portal_id").text
    if cert_type in context.data['network_and_storage'].keys():
        context.data[context.data['parents'][cert_type]][cert_type]['product_id'] \
            = context.browser.find_element_by_css_selector("div.col-sm-8.div_product_portal_id").text
    else:
        context.data[context.data['parents'][cert_type]][cert_type]['product_id'] \
            = context.browser.find_element_by_css_selector("div.col-sm-8.div_product_portal_id").text


@when('I accept certification policy document checkbox')
def step_impl(context):
    context.browser.find_element_by_id(
        "readPolicy").click()
