try:
    import xpath
    from behave import given, when, then
    from xml.dom import minidom
    import utilities,glob,gzip,re,time

    utilities.safe_selenium_imports(__file__, globals())
except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

@when('I validate the elements of this page')
def step_impl(context):
    progress_tab = context.browser.find_element_by_xpath(xpath.progress)
    if progress_tab.is_displayed():
        pass

    testing_tab = context.browser.find_element_by_xpath(xpath.testing)
    if testing_tab.is_displayed():
        pass

    dialog_tab = context.browser.find_element_by_xpath(xpath.dialog)
    if dialog_tab.is_displayed():
        pass

    details_tab = context.browser.find_element_by_xpath(xpath.details)
    if details_tab.is_displayed():
        pass

    related_cert_tab = context.browser.find_element_by_xpath(xpath.related_cert)
    if related_cert_tab.is_displayed():
        pass

    publish_tab = context.browser.find_element_by_xpath(xpath.publish)
    if publish_tab.is_displayed():
        pass

@then('The testing page is validated')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 15)

@when('I input "{make_name}" Make')
def step_impl(context, make_name):
    context.browser.find_element_by_id(
            "make").send_keys(str(make_name)+str(time.strftime("%Y-%m-%d %H:%M")))

@when('I input "{model_name}" Model')
def step_impl(context, model_name):
    context.browser.find_element_by_name(
            "product_name").send_keys(str(model_name)+str(time.strftime("%Y-%m-%d %H:%M")))

@when('I input "{product_url}" Product URL')
def step_impl(context, product_url):
    context.browser.find_element_by_name(
            "product_url").send_keys(product_url)

    #@when('I upload Product logo for hardware product')
#def step_impl(context):
#    And   I upload Product logo for hardware product

@when('I check the "{hwcert_test}" in test plan')
def step_impl(context,hwcert_test):
    ''' Func to check the test present in WUI test plan '''
    try:
        pattern = hwcert_test
        test_listing = context.browser.find_elements_by_xpath(xpath.test_plan_table)
        tests_list =  ([td.text for td in test_listing])
        # Removing unicode if present
        tests = [x.encode('utf-8') for x in tests_list]
        # Check if test is present
        for test in tests:
            if re.search(pattern,test):
                pass
    except Exception as e:
        utilities.printError(e, raiseError=True)


@when('I select the "{hwcert_test}" checkbox')
def step_impl(context,hwcert_test):
    ''' Func to select the test run the WUI'''
    test_select = context.browser.find_element_by_xpath(getattr(xpath,hwcert_test))
    test_select.click()


def get_sut_host(context,cert_type):
    try:
        sut_host = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
        return str(sut_host).strip()
    except Exception as e:
        utilities.printError(e, raiseError=True)


def read_xml_tag(context,tagname,cert_type):
    ''' Read the xml file with Tags '''
    try:
        sut = get_sut_host(context,cert_type)
        sut_xml_gz = glob.glob("/var/www/rhcert/store/systems/*"+sut+"*.xml.gz")
        filedata = gzip.open(sut_xml_gz[0],'rb')
        sut_sys_info = filedata.read()
        xmldoc  = minidom.parseString(sut_sys_info)
        data = xmldoc.getElementsByTagName(tagname)
        return data[0].firstChild.nodeValue
    except Exception as e:
        utilities.printError(e, raiseError=True)


def get_sut_info(context,cert_type):
    ''' Read the system info from registered SUT xml.gz '''
    try:
        sut = get_sut_host(context,cert_type)
        sut_xml_gz = glob.glob("/var/www/rhcert/store/systems/*"+sut+"*.xml.gz")
        filedata = gzip.open(sut_xml_gz[0],'rb')
        sut_sys_info = filedata.read()
        ip =  read_xml_tag(context,'ip-address',cert_type)
        arch = read_xml_tag(context,'arch',cert_type)
        hostname =  read_xml_tag(context,'hostname',cert_type)
        release =  read_xml_tag(context,'release',cert_type)
        product =  read_xml_tag(context,'product',cert_type)
        make = read_xml_tag(context,'make',cert_type)
        model = read_xml_tag(context,'model',cert_type)
        return {'arch':arch,'model':model,'make':make,'release':release,'product':product,'ip':ip,'hostname':hostname}
    except IndexError as e:
        print("Error in get_sut_info check Hardware:SUT or /var/www/rhcert/store/systems/")
        utilities.printError(e, raiseError=True)


@when('I click "{cert_type}" result link')
def step_impl(context,cert_type):
    ip = get_sut_info(context,cert_type)['ip']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[1]/a' % (ip_id)
    result_link = context.browser.find_element_by_xpath(find_link)
    result_link.click()


@given('I have a new "{cert_type}" certificate')
def step_impl(context,cert_type):
    ''' Read SUT system info and create a new hardware cert and product '''
    sys_info = get_sut_info(context,'hardware')
    arch = sys_info['arch']
    model = sys_info['model']+str(time.strftime("%Y-%m-%d %H:%M"))
    release = sys_info['release']
    product = sys_info['product']
    make = sys_info['make']+str(time.strftime("%Y-%m-%d %H:%M"))
    context.execute_steps(u"""
                                Given I am logged into RHCERT
                                When  I click on the New Certification button
                                When  I select product "%s"
                                And   I select RHEL version "%s" for "%s"
                                And   I select platform "%s"
                                And   I click on Next button
                                And   I click on New Product
                                And   I select vendor "Abbee (ID: 140)" for product creation
                                And   I select Program "Red Hat Enterprise Linux Certified Hardware"
                                And   I click on Next button
                                And   I input "%s" Make
                                And   I input "%s" Model
                                And   I input "http://www.redhat.com" Product URL
                                And   I click on Next button
                                Then  I see the "%s" product created on rhcert
                                Then  I see the "%s" certification created on rhcert against new Product
                                And   I click on Details button
                                And   I save the cert_nid of "%s" created on rhcert
                        """ % (product,release,cert_type,arch,make,model,cert_type,cert_type,cert_type))


@when('Test "{hwcert_test}" should pass')
def step_impl(context,hwcert_test):
    '''Check test run status'''
    status = context.browser.find_element_by_xpath(getattr(xpath,hwcert_test)).text
    if 	status == "PASS":
      pass
    else:
      print("Test Run Has FAILED")
      assert False,"The test failed"




@when('I click on SATA submit')
def step_impl(context):
    '''Click on submit button if present'''
    try:
        context.browser.find_element_by_xpath("//input[@type='submit']").click()
    except Exception as e:
        pass

@when('Check "{hwcert_test}" should pass')
def step_impl(context,hwcert_test):
    ''' Func to check the test status in WUI test plan '''
    try:
        test_count = 0
        pattern = hwcert_test
        test_listing = context.browser.find_elements_by_xpath(xpath.test_plan_table)
        tests_list =  ([td.text for td in test_listing])
        # Removing unicode if present
        tests = [x.encode('utf-8') for x in tests_list]
        # Check if test is present
        for test in tests:
            test_count+=1
            if re.search(pattern,test):
                xpath_test_status = str(xpath.test_plan_table)+"[%s]/td[4]/a" %(test_count)
                print (xpath_test_status)
                status = context.browser.find_elements_by_xpath(xpath_test_status).text
                if status == "PASS":
                  pass
                else:
                  utilities.printError(e, raiseError=True)
    except Exception as e:
        utilities.printError(e, raiseError=True)

@then('I see the "{cert_type}" result is submitted')
def step_impl(context, cert_type):
    ip = get_sut_info(context,cert_type)['ip']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[2]' % (ip_id)
    xpath = '//*[@id="%s"]/table/tbody/tr/td[2][contains(text(),"submitted")]' % (
        ip_id)
    webdriver.support.ui.WebDriverWait(context.browser, 1000).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, xpath)))
    result_link = context.browser.find_element_by_xpath(find_link)
    assert "submitted" in result_link.text


@when('I print the "{harness}" results')
def step_impl(context,harness):

    harness_list = ["cli","ci","rhcert-ci","rhcert-cli"]
    if ( harness not in harness_list ):
       assert False, "Harness not in list [cli ci rhcert-ci rhcert-cli]"

    if (harness == "ci" or harness == "rhcert-ci"):
       (exitStatus, stdout, stderr) = utilities.runShellCommand(
        "rhcert-ci print",
        rhcertCommand=False
       )
    elif(harness == "cli" or harness == "rhcert-cli"):
        (exitStatus, stdout, stderr) = utilities.runShellCommand(
        "rhcert-cli print",
        rhcertCommand=True
        )
    context.results = {
        "testExitStatus": exitStatus,
        "testOutput": stdout,
        "testErrorOutput": stderr,
    }

@given('The "{harness}" "{test}" test is planned')
def step_impl(context,harness,test):

    if (harness == "ci" or harness == "rhcert-ci"):
       planCmd = "rhcert-ci plan"
    elif(harness == "cli" or harness == "rhcert-cli"):
       planCmd = "rhcert-cli plan"
    testNames = None

    tests = utilities.runShellCommand(planCmd, rhcertCommand=True)

    if int(tests[0]) != 0:
        print("{0} did not complete successfully.".format(planCmd))
        assert False, "Unable to parse test plan information"

    # Else, the 'rhcert-ci or rhcert-cli plan' completed successfully.
    # Retrieve the list of planned tests by parsing the command's output.
    try:
        testNames = map(
            lambda test: test.split(' ')[0], tests[1].split('\n')
        )
    except (AttributeError,) as e:
        pass

    testPlanned = reduce(
        lambda x, y: x or y,
        map(
            lambda plannedTest: test in plannedTest,
            testNames
        ),
        False
    )
    print (testPlanned)
    if not testPlanned:
        context.running_scenario.skip("{0} was not planned on system {1}.".format(test, __hostname))

