try:
    import datetime
    import sys
    import time

    from behave import when, then, given
    from rhcert_cli import ssh_to_host

    import utilities
    utilities.safe_selenium_imports(__file__, globals())
except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.support.ui import WebDriverWait

@when('I click on create Openstack certification')
def step_impl(context):
    element = "a.openstack-logo.cert-logo > svg.cert-logo__image"
    context.browser.find_element_by_css_selector(element).click()


@then('I should see the openstack certification form')
def step_impl(context):
    src = context.browser.page_source
    assert 'Product Details' in src


@then('I should be on second form')
def step_impl(context):
    src = context.browser.page_source
    assert 'Create New OpenStack Entry' in src


@when('I fill all the mandatory details for "{plugin_type}" plugin')
def step_impl(context, plugin_type):
    common_steps(context)
    context.browser.find_element_by_id('button_next').click()
    submit_button = context.browser.find_element_by_xpath(
        "//button[@name='submit']")
    second_form(context, plugin_type)
    submit_button.click()


def common_steps(context):
    cert_name = 'ROSP9'
    suffix = datetime.datetime.now().strftime("%y%m%d_%H%M%S")
    cert_name = "_".join([cert_name, suffix])
    cert_version = context.browser.find_element_by_name('cert_version')
    cert_version.send_keys(cert_name)
    hostname = context.browser.find_element_by_name('hostname')
    hostname.send_keys('ROSP9.rhcert')
    source_url = context.browser.find_element_by_name('source_url')
    source_url.send_keys('http://www.example.com/source_url')
    blueprint_url = context.browser.find_element_by_name('blueprint_url')
    blueprint_url.send_keys('http://www.example.com/blueprint_url')
    install_guide = context.browser.find_element_by_name('install_guide')
    install_guide.send_keys('http://www.example.com/install_guide')


def second_form(context, plugin_type):
    if plugin_type == 'cinder':
        supported_services = context.browser.find_elements_by_xpath(
            "//input[contains(@name,'supported_service_')]")
        for supported_service in supported_services:
            supported_service.click()
        supported_protocol = context.browser.find_element_by_name(
            'supported_protocol')
        supported_protocol.send_keys('iSCSI')


@when('I click on back button')
def step_impl(context):
    back_button = context.browser.find_element_by_link_text('Back')
    back_button.click()


@when('I click on Run Interactive for openstack_verify')
def step_impl(context):
    interactive_button = context.browser.find_element_by_name(
        'interactive:openstack/openstack_verify')
    interactive_button.click()


@when('I click on Run Selected for rhel_verify')
def step_impl(context):
    rhel_verify_check_box = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='openstack_platform/rhel_verify']")
    rhel_verify_check_box.click()

    context.execute_steps(u"""
                          when I click on Run Selected Button
                          """)


@given('I have an "{plugin_type}" SUT')
def step_impl(context, plugin_type):
    context.execute_steps(u"""
                          given I have cleaned and verified RHCERT system
                          when I have sshed into "{plugin_type}" SUT
                          and Check if 'redhat-certification-openstack' package is installed
                          and "{plugin_type}" service is "running"
                          and I close SUT ssh connection
                        """.format(plugin_type=plugin_type))


@when('I have sshed into "{plugin_type}" SUT')
def step_impl(context, plugin_type):
    host = context.data[context.data['parents']
    [plugin_type]][plugin_type]['sut']['host']
    username = context.data[context.data['parents']
    [plugin_type]][plugin_type]['sut']['username']
    password = context.data[context.data['parents']
    [plugin_type]][plugin_type]['sut']['password']
    context.sut_ssh = ssh_to_host(host, username, password)


@when('I click on Run Interactive for openstack_scenarios')
def step_impl(context):
    interactive_button = context.browser.find_element_by_name(
        'interactive:openstack/openstack_scenarios')
    interactive_button.click()


@when('I click on Run Selected for Scap oval and xccdf Test')
def step_impl(context):
    scap_oval_check_box = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='openstack_scap/scap-oval-openstack']")
    scap_xccdf_check_box = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='openstack_scap/scap-xccdf-openstack']")
    scap_oval_check_box.click()
    scap_xccdf_check_box.click()

    context.execute_steps(u"""
                          when I click on Run Selected Button
                          """)


# ----- RCO 4.x ----

@when('I click on Run Interactive for "{cert_type}"')
def step_impl(context, cert_type):
    context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='openstack/supportable']").click()
    interactive_button = context.browser.find_element_by_css_selector(
        "input[type='checkbox'][value='openstack/%s']" % cert_type)
    interactive_button.click()


@when('I click on Run Selected')
def step_impl(context):
    run_selected = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Run Selected']")
    run_selected.click()


@when('I fill the tempest credentials for "{cert_type}"')
def step_impl(context, cert_type):
    host = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    host_username = context.data[context.data['parents'][cert_type]][cert_type]['sut']['username']
    host_password = context.data[context.data['parents'][cert_type]][cert_type]['sut']['password']

    try:
        client = ssh_to_host(host, username=host_username, password=host_password)
        stdin, stdout, stderr = client.exec_command('cat /root/keystonerc_admin')
        keystonerc_admin = {}
        keystonerc = stdout.read()
        if isinstance(keystonerc, bytes):
            keystonerc = keystonerc.decode("UTF-8")

        for i in keystonerc.split("\n"):
            if "=" in i:
                i = i.strip()
                keystonerc_admin[i.split("=")[0].split(" ")[1]] = i.split("=")[1]

    except:
        print("Error in connecting to the host.")
        sys.exit(1)

    webdriver.support.ui.WebDriverWait(context.browser, 1200).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, "//label[@for='prompt-answer']")))
    # time.sleep(5)  # send keystone_auth url
    auth_url = context.browser.find_element_by_id("prompt-answer")
    auth_url.clear()
    auth_url.send_keys(keystonerc_admin['OS_AUTH_URL'])
    submit = context.browser.find_element_by_xpath("//input[@type='submit']")
    time.sleep(5)
    submit.click()

    # For admin user name
    time.sleep(5)
    admin_username = context.browser.find_element_by_id("prompt-answer")
    admin_username.clear()
    admin_username.send_keys(keystonerc_admin['OS_USERNAME'])

    time.sleep(5)
    submit = context.browser.find_element_by_xpath("//input[@type='submit']")
    submit.click()

    # for admin password
    time.sleep(5)
    admin_password = context.browser.find_element_by_id("prompt-answer")
    admin_password.clear()
    admin_password.send_keys(keystonerc_admin['OS_PASSWORD'])

    time.sleep(5)
    submit = context.browser.find_element_by_xpath("//input[@type='submit']")
    submit.click()

    # for admin tenant name
    time.sleep(5)
    admin_tenant = context.browser.find_element_by_id("prompt-answer")
    admin_tenant.clear()
    admin_tenant.send_keys(keystonerc_admin['OS_TENANT_NAME'])

    time.sleep(5)
    submit = context.browser.find_element_by_xpath("//input[@type='submit']")
    submit.click()


def OSP_component_plugins():
    plugins_dict = {'cinder': ['qos', 'clone', 'snapshots', 'volume_types',
                               'quota-set-extension', 'backups',
                               'multiple-backends', 'availability',
                               'image_metadata', 'consistencygroups',
                               'volume_transfers', 'extensions', 'volumes',
                               'user_messages'],
                    'neutron': ['load_balancer', 'subnets', 'agent_management',
                                'qos', 'provider_networks', 'lbaas_listeners',
                                'service_type_management', 'lbaas_agent_scheduler',
                                'fwaas', 'metering', 'extra_dhcp_options', 'quotas',
                                'dhcp_agent_scheduler', 'port_security', 'trunk',
                                'lbaas_pools', 'load_balancers', 'networks',
                                'security_groups', 'revisions', 'subnetpools',
                                'flavors_extensions', 'l3_agent_scheduler',
                                'lbaas_members', 'versions', 'floating_ips',
                                'address_scopes', 'bgp', 'allowed_address_pair',
                                'routers', 'extensions', 'vpnaas', 'ports',
                                'lbaas_health_monitor'],
                    'sahara': ['cluster-templates', 'jobs', 'job-binaries',
                               'job-binary-internals', 'node-group-templates',
                               'data-sources', 'plugins'],
                    'manila': ['scheduler_stats', 'consistency_group',
                               'share_export_locations', 'share_manage', 'migration',
                               'share_instances', 'share_servers',
                               'availability_zones', 'snapshot_manage',
                               'admin_actions', 'quotas', 'shares', 'share_snapshot',
                               'multi_backend', 'share_networks', 'rules',
                               'shares_actions', 'services', 'security_services',
                               'microversions', 'limits', 'share_metadata',
                               'extensions', 'share_types']}
    return plugins_dict


@when('I select the "{cert_type}" plugins')
def step_impl(context, cert_type):
    plugins_dict = OSP_component_plugins()
    time.sleep(5)
    for i in plugins_dict[cert_type]:
        try:
            select_plugin = context.browser.find_element_by_css_selector(
                "input[type='checkbox'][value='%s']" % i)
            select_plugin.click()
        except:
            pass
    time.sleep(5)
    submit = context.browser.find_element_by_xpath("//input[@type='submit']")
    submit.click()


@when('I submit tempest_conf.json file')
def step_impl(context):
    time.sleep(5)
    submit = context.browser.find_element_by_xpath("//input[@type='submit']")
    submit.click()
