try:
    import os
    import re
    from random import choice
    from string import ascii_uppercase

    import paramiko
    from behave import given, when, then

    import utilities
    utilities.safe_selenium_imports(__file__, globals())
except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.support.ui import WebDriverWait, Select

@given('I am on RHCERT login page')
def step_impl(context):
    context.browser.get("http://" + context.data['common']['rhcert']['ip'])


@then('I should be logged in to RHCERT')
def step_impl(context):
    accountUserName = context.browser.find_element_by_class_name(
        'dropdown').text
    assert accountUserName == context.data['common']['rhcert']['sso_username']


@when('I fill in the rhcert login credentials')
def step_impl(context):
    username = context.browser.find_element_by_id('username')
    password = context.browser.find_element_by_id('password')
    username.send_keys(context.data['common']['rhcert']['sso_username'])
    password.send_keys(context.data['common']['rhcert']['sso_password'])


@when('I click on Log In button')
def step_impl(context):
    login = context.browser.find_element_by_name("login")
    login.click()

@when('I click on Cancel button')
def step_impl(context):
    cancel = context.browser.find_element_by_name("opt_out_login")
    cancel.click()

@when('I click on CI Sandboxes')
def step_impl(context):
    context.browser.find_element_by_link_text("CI Sandboxes").click()


@when('I click on New Sandbox button')
def step_impl(context):
    login = context.browser.find_element_by_name("next-button")
    login.click()


@when('I select the RHEL "{version}" version')
def step_impl(context):
    version = webdriver.support.ui.Select(context.browser.find_element_by_type("button"))
    version.select_by_visible_text(version)
    context.browser.find_element_by_name("next").click()


@then('I should see the CI certification created in the list')
def step_impl(context):
    login = context.browser.find_element_by_name("new")


@when("I select 'Container, Cloud, or RHELOSP'")
def step_impl(context):
    cert_type = context.browser.find_element_by_css_selector(
        "input[type='radio'][value='sso']")
    cert_type.click()


@when("I click on 'login' icon")
def step_impl(context):
    icon = context.browser.find_element_by_link_text("login")
    icon.click()


@given('I am logged in to RHCERT')
def step_impl(context):
    context.execute_steps(u"""
                          given I am on RHCERT login page
                          when I click on 'login' icon
                          and I fill in the rhcert login credentials
                          and I click on Log In button
                          """)


@when("I click on Add Products")
def step_impl(context):
    add_product = context.browser.find_element_by_name("add")
    add_product.click()


@when("I go to configuration tab")
def step_impl(context):
    context.browser.find_element_by_css_selector(
        "a[href='/configuration'][title='Server settings']").click()


@when('I add the "{cert_type}" SUT')
def step_impl(context, cert_type):
    sut = context.browser.find_element_by_id('new-system')
    sut.send_keys(get_sut_host(context, cert_type))
    context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Add']").click()


@then('"{cert_type}" SUT should be registered successfully')
def step_impl(context, cert_type):
    assert context.browser.find_element_by_link_text(
        get_sut_host(context, cert_type))

    # This is to go back to home page
    context.browser.find_elements_by_tag_name("img")[0].click()


@given('I have registered the "{cert_type}" SUT with RHCERT')
def step_impl(context, cert_type):
    context.execute_steps(u"""
                          When I go to configuration tab
                          And I add the "%s" SUT
                          Then "%s" SUT should be registered successfully
                          """ % (cert_type, cert_type))


@when('I select the "{cert_type}" radio button')
def step_impl(context, cert_type):
    if cert_type in context.data['netwrk_and_storage'].keys():
        cert_type = 'openstack'
    input_str = "input[type = 'radio'][value = '%s']" % cert_type
    prod_type = context.browser.find_element_by_css_selector(input_str)
    prod_type.click()


@when('I click on Next')
def step_impl(context):
    click_next = context.browser.find_element_by_name("next")
    click_next.click()


@when('I search for "{cert_type}" product')
def step_impl(context, cert_type):
    node_id = get_node_id(context, cert_type)
    search(context, node_id)


@when('I select a product from the list')
def step_impl(context):
    context.browser.find_element_by_name("product-id").click()


@when('I click on Add Selected Products')
def step_impl(context):
    click_next = context.browser.find_element_by_name("add-selected")
    click_next.click()


@then('I can see the "{cert_type}" product on the home page')
def step_impl(context, cert_type):
    node_id = get_node_id(context, cert_type)
    find_node_id = "//td[contains(text(),%s)]" % node_id
    index_node_id = context.browser.find_element_by_xpath(find_node_id)
    assert int(index_node_id.text) == node_id


@given('I have "{cert_type}" product available')
def step_impl(context, cert_type):
    node_id = get_node_id(context, cert_type)
    search(context, node_id)
    product_name = context.browser.find_element_by_xpath(
        "//table/tbody/tr/td/a").text
    product_name_link = context.browser.find_element_by_link_text(product_name)
    product_name_link.click()


@when('I click on a "{cert_type}" certification')
def step_impl(context, cert_type):
    cert_id = get_cert_id(context, cert_type)
    search(context, cert_id)
    certification_name = context.browser.find_element_by_class_name(
        "sorting_1")
    certification_name.click()


@then('I should see the test plan for the "{cert_type}" certification')
def step_impl(context, cert_type):
    table = context.browser.find_element_by_xpath("//form/table/tbody[2]")
    row_count = (len(table.find_elements_by_tag_name("tr")))
    if cert_type == "cloud":
        assert row_count == 4
    else:
        assert row_count == 5


@when("I click on Continue Testing button")
def step_impl(context):
    context.browser.find_element_by_name("continue").click()


@when("I click on Run Selected Button")
def step_impl(context):
    result_type = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Run Selected']")
    result_type.click()


def search(context, product):
    search = context.browser.find_element_by_css_selector(
        "input[type=\"search\"]")
    search.clear()
    search.send_keys(product)


def get_node_id(context, cert_type):
    product_id = context.data[context.data['parents'][cert_type]][cert_type]['product_id']
    return product_id


def get_cert_id(context, cert_type):
    cert_id = context.data[context.data['parents'][cert_type]][cert_type]['cert_id']
    return cert_id


def get_sut_host(context, cert_type):
    sut_host = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    return sut_host


@when('I click Results tab')
def step_impl(context):
    testing_page = context.browser.find_element_by_link_text('Results')
    testing_page.click()


@when('I click on the Submit Results button')
def step_impl(context):
    test_button = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Submit Results']")
    test_button.click()


@when('I click on the Testing tab')
def step_impl(context):
    testing_page = context.browser.find_element_by_link_text('Testing')
    testing_page.click()


@when('I click on the System button')
def step_impl(context):
    context.browser.find_element_by_partial_link_text('System').click()


@when('I wait for a while')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 100).until(
        webdriver.support.expected_conditions.presence_of_element_located((webdriver.common.by.By.NAME, "continue")))


@then('I should see the Continue Testing button')
def step_impl(context):
    continue_testing_button = context.browser.find_element_by_name("continue")
    continue_testing_button.is_enabled()


@when('I select the radio button for the "{cert_type}" SUT')
def step_impl(context, cert_type):
    table = context.browser.find_element_by_xpath("//form/table/tbody")
    sut_host = context.data[context.data['parents']
    [cert_type]][cert_type]['sut']['host']
    for tr in table.find_elements_by_tag_name("tr"):
        if sut_host in tr.text:
            system_under_test_page = tr.find_element_by_css_selector(
                "input[type='radio'][name='system-path']")
            system_under_test_page.click()


@when('I click on the Test button')
def step_impl(context):
    test_button = context.browser.find_element_by_css_selector(
        "button[type='submit'][value='Test']")
    test_button.click()


@when('I wait for a while for "{cert_type}" tests to run')
def step_impl(context, cert_type):
    webdriver.support.ui.WebDriverWait(context.browser, 7200).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, '//*[@id="-continue"]/table/tbody/tr/td[1][contains(text(),\
            "Finished test run")]')))


@when('I click on Results Tab')
def step_impl(context):
    results_page = context.browser.find_element_by_link_text('Results')
    results_page.click()


@when('I click the "{cert_type}" results link in the table')
def step_impl(context, cert_type):
    ip = context.data[context.data['parents']
    [cert_type]][cert_type]['sut']['host']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[1]/a' % (ip_id)
    result_link = context.browser.find_element_by_xpath(find_link)
    result_link.click()


@when('I submit the results')
def step_impl(context):
    submit_button = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Submit Results']")
    submit_button.click()
    final_submit_button = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Submit']")
    final_submit_button.click()


@given("I have cleaned and verified RHCERT system")
def step_impl(context):
    context.execute_steps(u"""
                           given I have sshed into RHCERT
                           and 'redhat-certification' package is installed
                           and 'redhat-certification-backend' package is installed
                           when "httpd" service is "running"
                           and "firewalld" service is "inactive"
                           and I stop the rhcertd service
                           and  I clean the RHCERT system
                           and  I start the rhcertd service
                           and  I close the RHCERT ssh connection
                           """)


@given("I have a cloud SUT")
def step_impl(context):
    context.execute_steps(u"""
                          given I have cleaned and verified RHCERT system
                          when I have sshed into cloud SUT
                          and Check if 'redhat-certification-cloud' package is installed
                          and Check if 'secret-sos' package is installed
                          and Check if 'redhat-certitication-commons' package is installed
                          and I close SUT ssh connection
                        """)


def test_setup(context, vagrant_box):
    deploy_box(context, vagrant_box)


def get_source_path(vagrant_box):
    """
    This function will return the absolute path with respect to SUT
    """
    local_path = os.getcwd()
    path_dict = {
        'cloud': 'vagrantfiles/sut-cloud/Vagrantfile',
        'container': 'vagrantfiles/sut-container/Vagrantfile',
        'cinder': 'vagrantfiles/sut-openstack-cinder/Vagrantfile',
        'neutron': 'vagrantfiles/sut-openstack-neutron/Vagrantfile',
        'sahara': 'vagrantfiles/sut-openstack-sahara/Vagrantfile',
        'manila': 'vagrantfiles/sut-openstack-manila/Vagrantfile',
        'management': 'vagrantfiles/sut-openstack-management/Vagrantfile',
        'rhcert': 'vagrantfiles/rhcert/Vagrantfile'}
    path = os.path.join(local_path, path_dict[vagrant_box])
    return path


def deploy_box(context, vagrant_box):
    vm_host_username = "jenkins"
    vm_host_password = ""
    vm_host_ip = "virt.swcert.cee.pnq.redhat.com"
    vm_host_keyfile = '~/.ssh/id_rsa'
    privatekeyfile = os.path.expanduser('~/.ssh/id_rsa')
    try:
        mykey = paramiko.RSAKey.from_private_key_file(privatekeyfile)
    except IOError as e:
        print('Unable to find SSH keyfile: %s') % str(e)
        raise
    transport = paramiko.Transport((vm_host_ip, 22))
    try:
        transport.connect(username=vm_host_username, pkey=mykey)
    except paramiko.AuthenticationException as e:
        print("Unable to logon - are you sure you've added the pubkey to \
            the server?: %s") % str(e)
        raise
    sftp = paramiko.SFTPClient.from_transport(transport)
    source_path = get_source_path(vagrant_box)
    random_path = ''.join(choice(ascii_uppercase) for i in range(6))
    path = vagrant_box + random_path
    dest_path = os.path.join("/tmp/spicegate/", path)
    if vagrant_box == 'rhcert':
        context.data['common']['rhcert']['path'] = str(dest_path).strip()
    else:
        context.data[context.data['parents'][vagrant_box]][vagrant_box]['sut']['path'] = str(dest_path)
    try:
        dirs = sftp.listdir("/tmp")
        if "spicegate" not in dirs:
            sftp.mkdir("/tmp/spicegate", mode=0o777)
        dirs = sftp.listdir("/tmp/spicegate")
        if vagrant_box not in dirs:
            sftp.mkdir(dest_path, mode=0o777)
    except IOError as e:
        print("Not able to create destination directory %s") % str(e)
        raise
    try:
        if vagrant_box == 'rhcert':
            sftp.put(source_path, context.data['common']['rhcert']['path'] + '/Vagrantfile')
        else:
            sftp.put(source_path,
                     context.data[context.data['parents'][vagrant_box]][vagrant_box]['sut']['path'] + '/Vagrantfile')
    except IOError as e:
        print("Unable to copy vagrant files %s to remote directory %s") % (
            source_path, str(dest_path))
        raise
    sftp.close()
    command = "cd %s ; vagrant up --no-provision ; vagrant provision" % dest_path
    print(command)
    handle = paramiko.SSHClient()
    handle.load_system_host_keys()
    handle.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    try:
        handle.connect(
            vm_host_ip,
            username=vm_host_username,
            key_filename=privatekeyfile)
    except paramiko.SSHException:
        print("Connection Failed")
        raise
    stdin, stdout, stderr = handle.exec_command(command)
    while not stdout.channel.exit_status_ready():
        pass
    exit_code = stdout.channel.recv_exit_status()
    if exit_code:
        for line in stdout.readlines():
            print(line.strip())
        for line in stderr.readlines():
            print(line.strip())
        print('SUT Deploy failed')
        return False
    for line in stdout.readlines():
        line = line.strip()
        if re.search("IPADDRESS:", line):
            ipline = line.split(':')
            ip = ipline[2]
            print("Sut IP Address = %s" % str(ip).strip())
            if vagrant_box == 'rhcert':
                context.data['common']['rhcert']['ip'] = str(ip).strip()
                context.data['common']['rhcert']['username'] = 'vagrant'
                context.data['common']['rhcert']['password'] = 'vagrant'
            else:
                context.data[context.data['parents'][vagrant_box]][
                    vagrant_box]['sut']['host'] = str(ip).strip()
                context.data[context.data['parents'][vagrant_box]][
                    vagrant_box]['sut']['username'] = 'vagrant'
                context.data[context.data['parents'][vagrant_box]][
                    vagrant_box]['sut']['password'] = 'vagrant'


def destroy_box(context, vagrant_box):
    print("SUT destroy")
    vm_host_username = "jenkins"
    vm_host_password = ""
    vm_host_ip = "virt.swcert.cee.pnq.redhat.com"
    vm_host_keyfile = '~/.ssh/id_rsa'
    print("Destroying %s" % vagrant_box)
    if vagrant_box == 'rhcert':
        command = "cd %s; vagrant destroy" % (
            context.data['common']['rhcert']['path'])
    else:
        command = "cd %s; vagrant destroy" % (
            context.data[context.data['parents'][vagrant_box]][vagrant_box]['sut']['path'])
    privatekeyfile = os.path.expanduser('~/.ssh/id_rsa')
    client = paramiko.SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    try:
        client.connect(
            vm_host_ip,
            username=vm_host_username,
            key_filename=privatekeyfile)
    except paramiko.SSHException:
        print("Connection Failed")
        raise

    stdin, stdout, stderr = client.exec_command(command)
    while not stdout.channel.exit_status_ready():
        pass
    exit_code = stdout.channel.recv_exit_status()
    if exit_code:
        for line in stdout.readlines():
            print(line.strip())
        for line in stderr.readlines():
            print(line.strip())
    print('Box destroyed')
    if vagrant_box == 'rhcert':
        dir_name = context.data['common']['rhcert']['path']
    else:
        dir_name = context.data[context.data['parents'][vagrant_box]][vagrant_box]['sut']['path']
    command = "rm -rf %s" % dir_name
    stdin, stdout, stderr = client.exec_command(command)
    while not stdout.channel.exit_status_ready():
        pass
    exit_code = stdout.channel.recv_exit_status()
    if exit_code:
        for line in stdout.readlines():
            print(line.strip())
        for line in stderr.readlines():
            print(line.strip())
    print("file removed")


@when("I have sshed into cloud SUT")
def step_impl(context):
    host = context.data['platform']['cloud']['sut']['host']
    username = context.data['platform']['cloud']['sut']['username']
    password = context.data['platform']['cloud']['sut']['password']
    context.sut_ssh = ssh_to_host(host, username, password)


@when("Check if '{package}' package is installed")
def step_impl(context, package):
    cmd = "yum -y install %s | wc -l" % package
    stdin, stdout, stderr = context.sut_ssh.exec_command(cmd)
    assert (len(stdout.readlines()) > 0) is True


@when("I close SUT ssh connection")
def step_impl(context):
    context.sut_ssh.close()


@then('I can see the "{cert_type}" logs are submitted')
def step_impl(context, cert_type):
    ip = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[3]' % (ip_id)
    xpath = '//*[@id="%s"]/table/tbody/tr/td[3][contains(text(),"submitted")]' % (
        ip_id)
    webdriver.support.ui.WebDriverWait(context.browser, 3000).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, xpath)))
    result_link = context.browser.find_element_by_xpath(find_link)
    assert "submitted" in result_link.text


@when('I am on the test plan for the created "{cert_type}" certification \
       from rhcert')
def step_impl(context, cert_type):
    cert_id = get_rhcert_cert_id(context, cert_type)
    search(context, cert_id)
    certification_name = context.browser.find_element_by_class_name(
        "sorting_1")
    certification_name.click()


def get_rhcert_cert_id(context, cert_type):
    cert_id = context.data[context.data['parents'][cert_type]][cert_type]['cert_id_rhcert']
    return cert_id

def get_product_id(context, cert_type):
    product_id = context.data[context.data['parents'][cert_type]][cert_type]['product_id_rhcert']
    return product_id

def get_product_name(context, cert_type):
    product_name = context.data[context.data['parents'][cert_type]][cert_type]['product_name_rhcert']
    return product_name
