try:
    import time
    import urllib2
    import os
    import shutil
    import xpath

    from behave import given, when, then

    import utilities
    utilities.safe_selenium_imports(__file__, globals())
except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.support.ui import WebDriverWait, Select

@given('I go to RHCERT login page')
def step_impl(context):
    context.browser.get("http://" + context.data['common']['rhcert']['ip'])

@when('I fill in the rhcert login credential')
def step_impl(context):
    username = context.browser.find_element_by_id('username')
    password = context.browser.find_element_by_id('password')
    username.send_keys(context.data['common']['rhcert']['sso_username'])
    password.send_keys(context.data['common']['rhcert']['sso_password'])

@when('I click the Login button')
def step_impl(context):
    login = context.browser.find_element_by_name("login")
    login.click()

@then('I should be logged into RHCERT')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 300).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, "//button[contains(.,'New Certification')]")))
    accountUserName = context.browser.find_element_by_class_name(
        'dropdown').text
    assert accountUserName == context.data['common']['rhcert']['sso_username']

@given('I am logged into RHCERT')
def step_impl(context):
    context.execute_steps(u"""
                          given I go to RHCERT login page
                          when I fill in the rhcert login credential
                          and I click the Login button
                          then I should be logged into RHCERT
                          And  I refresh the WUI
                          """)

@given('Open "{cert_type}" certification')
def step_impl(context,cert_type):
    nid = context.data[context.data['parents'][cert_type]][cert_type]['cert_id_rhcert']
    context.execute_steps(u"""
                          given I go to RHCERT login page
                          when I fill in the rhcert login credential
                          and I click the Login button
                          then I should be logged into RHCERT
                          And  I refresh the WUI
                          And  I search for  "{0}" cert_nid
                          """.format(nid))

@given('I am logged into RHCERT in Management mode')
def step_impl(context):
    context.execute_steps(u"""
                          given I go to RHCERT login page
                          when I fill in the rhcert login credential
                          and I click the Login button
                          then I should be logged into RHCERT
                          """)


@when("I click on Add Product")
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 200).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.NAME, 'adding')))

    add_product = context.browser.find_element_by_name("adding")
    add_product.click()


@then('I refresh the WUI')
def step_impl(context):
    checking_refresh_button(context)
    wait_till_updating(context)
@when('I select "{cert_type}" radio button')
def step_impl(context, cert_type):
    if cert_type in context.data['network_and_storage'].keys():
        cert_type = 'openstack'
    input_str = "input[type = 'radio'][value = '%s']" % cert_type
    prod_type = context.browser.find_element_by_css_selector(input_str)
    prod_type.click()

@when('I click on Next button')
def step_impl(context):
    next_button = context.browser.find_element_by_name("next-button")
    next_button.click()

@when('I click on Next button on New Sandbox page')
def step_impl(context):
    next_button = context.browser.find_element_by_name("next-button")
    next_button.click()

@when('I select a product from the products list')
def step_impl(context):
    context.browser.find_element_by_name("product-id").click()

@when('I click on Add Selected Product')
def step_impl(context):
    click_next = context.browser.find_element_by_name("add-selected")
    click_next.click()

@when('I search for the "{product}"')
def step_impl(context, product):
    webdriver.support.ui.WebDriverWait(context.browser, 200).until(
        webdriver.support.expected_conditions.visibility_of_element_located((webdriver.common.by.By.CSS_SELECTOR,
                                          "input[type=\"search\"]")))
    search = context.browser.find_element_by_css_selector(
        "input[type=\"search\"]")
    search.clear()
    search.send_keys(product)

@when('I click on the "{product}"')
def step_impl(context, product):
    product_name = context.browser.find_element_by_partial_link_text(product)
    # product_name = context.browser.find_element_by_link_text(product)
    product_name.click()

@when('I click on Sandboxes')
def step_impl(context):
    context.browser.find_element_by_css_selector(
        "a[href='/sandboxes']").click()

@when('I click on add Certification')
def step_impl(context):
    context.browser.find_element_by_name("adding").click()
    webdriver.support.ui.WebDriverWait(context.browser, 200)


@when('I create "{cert_version}" certification from rhcert')
def step_impl(context, cert_version):
    context.browser.find_element_by_name(
        "image_name").send_keys("create_cert")
    version = webdriver.support.ui.Select(context.browser.find_element_by_name("program_version"))
    version.select_by_visible_text(cert_version)
    context.browser.find_element_by_name("next").click()

@when('I could see the "{cert_type}" certification created on rhcert')
def step_impl(context, cert_type):
    webdriver.support.ui.WebDriverWait(context.browser, 360).until(webdriver.support.expected_conditions.visibility_of_element_located((webdriver.common.by.By.XPATH, "//a[contains(.,'Dialog')]")))
    element = context.browser.find_element_by_xpath(xpath.cert_create_alert)
    success_message = element.text
    certification_id = success_message.split(' ')[10]
    context.data[context.data['parents'][cert_type]][cert_type]['cert_id_rhcert'] = certification_id

@when('I click "{cert_type}" cert id')
def step_impl(context, cert_type):
    search = context.browser.find_element_by_css_selector("input[type=\"search\"]")
    search.clear()
    search.send_keys(context.data[context.data['parents'][cert_type]][cert_type]['cert_id_rhcert'])
    search_link = context.browser.find_element_by_partial_link_text("Red Hat")
    search_link.click()

@when('I click on "{program}" Sandbox id')
def step_impl(context, program):
    search_link = context.browser.find_element_by_partial_link_text("Red Hat")
    search_link.click()

@given('I have registered "{cert_type}" SUT with RHCERT')
def step_impl(context, cert_type):
    context.execute_steps(u"""
                          When I go to the configuration tab
                          And I add a "%s" SUT
                          Then The "%s" SUT should be registered successfully
                          """ % (cert_type, cert_type))

@when("I go to the configuration tab")
def step_impl(context):
    context.browser.find_element_by_css_selector(
        "a[href='/configuration'][title='Server settings']").click()

@when('I add a "{cert_type}" SUT')
def step_impl(context, cert_type):
    sut = context.browser.find_element_by_id('new-system')
    sut.send_keys(get_sut_host(context, cert_type))
    context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Add']").click()

@then('The "{cert_type}" SUT should be registered successfully')
def step_impl(context, cert_type):
    assert context.browser.find_element_by_link_text(
        get_sut_host(context, cert_type))

    # This is to go back to home page
    #context.browser.find_elements_by_tag_name("img")[0].click()

def get_sut_host(context, cert_type):
    sut_host = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    return str(sut_host).strip()

@when('I click on Testing tab')
def step_impl(context):
    testing_page = context.browser.find_element_by_link_text('Testing')
    testing_page.click()

@when('I click on Add System button')
def step_impl(context):
    context.browser.find_element_by_partial_link_text('System').click()

@when('I check for override button')
def step_impl(context):
    try:
        override_button = context.browser.find_element_by_name('override')
        if override_button.is_displayed():
            override_button.click()
    except:
        pass

@when('I select the radio button for "{cert_type}" SUT')
def step_impl(context, cert_type):
    table = context.browser.find_element_by_xpath(xpath.add_sut_table)
    sut_host = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    for tr in table.find_elements_by_tag_name("tr"):
        if sut_host in tr.text:
            webdriver.support.ui.WebDriverWait(context.browser, 200).until(
                webdriver.support.expected_conditions.visibility_of_element_located((
                    webdriver.common.by.By.NAME, 'system-path')))

            # system_under_test_page = tr.find_element_by_css_selector(
            #    "input[type='radio'][name='system-path']")
            system_under_test_page = context.browser.find_element_by_name('system-path')
            system_under_test_page.click()

@when('I click on Test button')
def step_impl(context):
    test_button = context.browser.find_element_by_css_selector(
        "button[type='submit'][value='Test']")
    test_button.click()

@when('I wait for a while for test verification')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 300).until(
        webdriver.support.expected_conditions.presence_of_element_located((webdriver.common.by.By.NAME, "continue")))

@when('I see test plan for "{cert_type}" certification')
def step_impl(context, cert_type):
    table = context.browser.find_element_by_xpath("//*[@class='table table-striped table-bordered table-hover']/tbody")
    row_count = (len(table.find_elements_by_tag_name("tr")))
    if cert_type == "cloud":
        assert row_count == 4
    else:
        assert row_count == 5

@when("I click on the Continue Testing button")
def step_impl(context):
    context.browser.find_element_by_name("continue").click()

@when('I wait for a while for the "{cert_type}" tests to run')
def step_impl(context, cert_type):
    webdriver.support.ui.WebDriverWait(context.browser, 8200).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, '//*[@id="-continue"]/table/tbody/tr/td[1][contains(text(),\
            "Finished test run")]')))
    time.sleep(5)

@when('I click the "{cert_type}" result link')
def step_impl(context, cert_type):
    ip = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[1]/a' % (ip_id)
    result_link = context.browser.find_element_by_xpath(find_link)
    result_link.click()

@when('I submit the result')
def step_impl(context):
    submit_results = context.browser.find_element_by_xpath(xpath.submit_results_button)
    submit_results.click()
    time.sleep(10)
    submit = context.browser.find_element_by_xpath(xpath.submit_button)
    submit.click()

@then('I can see the "{cert_type}" result is submitted')
def step_impl(context, cert_type):
    ip = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[2]' % (ip_id)
    xpath = '//*[@id="%s"]/table/tbody/tr/td[2][contains(text(),"submitted")]' % (
        ip_id)
    webdriver.support.ui.WebDriverWait(context.browser, 1000).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, xpath)))
    result_link = context.browser.find_element_by_xpath(find_link)
    assert "submitted" in result_link.text

@when('I click add local product')
def step_impl(context):
    local_product = context.browser.find_element_by_xpath("(//button[@name='adding'])[2]")
    local_product.click()

@when('I create a local product "{product}"')
def step_impl(context, product):
    vendor_name = context.browser.find_element_by_xpath("//input[@name='vendor']")
    vendor_name.clear()
    vendor_name.send_keys("Abbee")
    product_name = context.browser.find_element_by_xpath("//input[@name='product']")
    product_name.clear()
    product_name.send_keys(product)
    add_product = context.browser.find_element_by_xpath(
        "//input[@name='adding'][@value='Add Product']")
    add_product.click()

@when('I search for local "{product}"')
def step_impl(context, product):
    search = context.browser.find_element_by_xpath("(//input[@type='search'])[2]")
    search.clear()
    search.send_keys(product)

@when('I click certification option')
def step_impl(context):
    certification = context.browser.find_element_by_xpath(
        "//button[@title='Red Hat Enterprise Linux 7.3']")
    certification.click()

@when('I create "{certification}" for local product')
def step_impl(context, certification):
    versions = context.browser.find_elements_by_xpath("//span[@class='text']")
    for version in versions:
        rhel = version.text
        if rhel == certification:
            version.click()

@then('I click on the Next button')
def step_impl(context):
    click_next = context.browser.find_element_by_name("adding")
    click_next.click()

@when('I click the certification created')
def step_impl(context):
    ID = context.browser.find_element_by_xpath("//table[@id='DataTables_Table_0']//th[3]")
    ID.click()
    ID.click()
    local_cert = context.browser.find_element_by_partial_link_text("Red Hat")
    local_cert.click()

@then('A local product "{name}" is created')
def step_impl(context, name):
    context.browser.find_element_by_partial_link_text(name)

@then('I have the "{cert_type}" result link')
def step_impl(context, cert_type):
    ip = context.data[context.data['parents'][cert_type]][cert_type]['sut']['host']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr/td[1]/a' % (ip_id)
    result_link = context.browser.find_element_by_xpath(find_link)
    result_link.click()

@when('I click on Download Button in the Submitted Test Results table')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 1800).until(
            webdriver.support.expected_conditions.visibility_of_element_located((
                webdriver.common.by.By.XPATH, '//a[@class="btn btn-primary"][contains(.,"Download")]')))
    download_button = context.browser.find_element_by_xpath('//a[@class="btn btn-primary"][contains(.,"Download")]')
    if download_button.is_displayed():
        download_button.click()

@then('The Results File is downloaded')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 1800).until(
            webdriver.support.expected_conditions.visibility_of_element_located((
                webdriver.common.by.By.XPATH, '//table[@class="table table-striped table-bordered table-hover"]')))

@when('I check for Refresh Button')
def step_impl(context):
    checking_refresh_button(context)

def checking_refresh_button(context):
    current_status = context.browser.find_element_by_xpath(xpath.refresh_button)
    if current_status.text == "Updating...":
        pass
    else:
        context.browser.find_element_by_name("refresh").click()

@then('I can see the Refresh Button')
def step_impl(context):
    wait_till_updating(context)

def wait_till_updating(context):
    try:
        webdriver.support.ui.WebDriverWait(context.browser, 1800).until(
                webdriver.support.expected_conditions.visibility_of_element_located((
                webdriver.common.by.By.XPATH, '//*[@id="cwe-sync-refresh"][contains(text(),\
                        "Refresh")]')))

    except:
        configuration = context.browser.find_element_by_xpath(xpath.configuration_link)
        configuration.click()
        try:
            reset_button = context.browser.find_element_by_name('reset-sync')
            if reset_button.is_displayed():
                reset_button.click()
        except:
            pass

    finally:
        main_page = context.browser.find_element_by_xpath(xpath.main_page_img)
        main_page.click()
        webdriver.support.ui.WebDriverWait(context.browser, 1800).until(
                webdriver.support.expected_conditions.visibility_of_element_located((
                webdriver.common.by.By.XPATH, '//*[@id="cwe-sync-refresh"][contains(text(),\
                        "Refresh")]')))

attachment_dir = "/tmp/attachments/"

@when('I add an attachment')
def step_impl(context):
    if os.path.isdir(attachment_dir) == False:
        os.mkdir(attachment_dir)
    response = urllib2.urlopen(context.data['common']['attachmentpath'])
    with open(context.data['common']['attachmentfile'], 'w') as at:
        at.write(response.read())
    context.browser.find_element_by_xpath("//input[@id='chosenFile']").send_keys(context.data['common']['attachmentfile'])
    time.sleep(5)

@when('I add description "{description}" for the attachment')
def step_impl(context, description):
    context.browser.find_element_by_xpath("//input[@id='description']").send_keys(description)
    time.sleep(5)

@then('I see attachment is successfully uploaded')
def step_impl(context):
    if os.path.isdir(attachment_dir) == True:
        shutil.rmtree(attachment_dir)
    webdriver.support.ui.WebDriverWait(context.browser, 8200).until(webdriver.support.expected_conditions.visibility_of_element_located((webdriver.common.by.By.XPATH, "//div[contains(@class,'toast-pf alert alert-success alert-dismissible')]")))

@when('I enter "{supp_case_id}" Support Case ID')
def step_impl(context, supp_case_id):
    context.browser.find_element_by_xpath("//input[@name='support_case_id']").send_keys(supp_case_id)

@when('I select "{kbase_type}" type')
def step_impl(context, kbase_type):
    kbase_option = webdriver.support.ui.Select(context.browser.find_element_by_name('kbase_type'))
    kbase_option.select_by_visible_text(kbase_type)

@when('I enter "{kbase_id}" KBase Reference ID')
def step_impl(context, kbase_id):
    context.browser.find_element_by_xpath("//input[@name='kbase_id']").send_keys(kbase_id)

@when('I click on update button to update certification details')
def step_impl(context):
    context.browser.find_element_by_xpath("//button[@class='login_required btn btn-primary btn-lg']").click()

@then('I can see the Certification details updated')
def step_impl(context):
    webdriver.support.ui.WebDriverWait(context.browser, 8200).until(webdriver.support.expected_conditions.visibility_of_element_located((webdriver.common.by.By.XPATH, "//div[contains(@class,'toast-pf alert alert-success alert-dismissible')]")))

@when('I select Make "{make_name}"')
def step_impl(context, make_name):
    make = webdriver.support.ui.Select(context.browser.find_element_by_id("vendor_make"))
    make.select_by_visible_text(make_name)

@when('I select Name "{product_name}"')
def step_impl(context, product_name):
    product_name_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('vendor_product_id'))
    product_name_option_link.select_by_visible_text(product_name)

# ---- Openstack Certification Code ---- #

@when('I click on "{tab}" tab')
def step_impl(context, tab):
    time.sleep(5)
    product_name = context.browser.find_element_by_partial_link_text(tab)
    product_name.click()

@when('I select the "{cert_type}" openstack certification')
def step_impl(context, cert_type):
    certification = context.browser.find_element_by_xpath("//button[@title='new']")
    certification.click()
    options = context.browser.find_elements_by_xpath("//span[@class='text']")
    for option in options:
        select = option.text
        if select == cert_type:
            option.click()

@when('I select the current "{cert_type}" test run and submit the results')
def step_impl(context, cert_type):
    host = get_sut_host(context, cert_type)
    time.sleep(5)
    link = context.browser.find_element_by_xpath("//*[@id='" + host + "-results']/table/tbody/tr[1]/td[1]/a")
    link.click()
    time.sleep(5)
    submit_button = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Submit Results']")
    submit_button.click()
    time.sleep(2)
    final_submit_button = context.browser.find_element_by_css_selector(
        "input[type='submit'][value='Submit']")
    final_submit_button.click()

@then('I verify the "{cert_type}" logs are submitted')
def step_impl(context, cert_type):
    ip = context.data[context.data['parents']
    [cert_type]][cert_type]['sut']['host']
    ip_id = "%s-results" % (ip)
    find_link = '//*[@id="%s"]/table/tbody/tr[1]/td[2]' % (ip_id)
    xpath = '//*[@id="%s"]/table/tbody/tr[1]/td[2][contains(text(),"submitted")]' % (
        ip_id)
    webdriver.support.ui.WebDriverWait(context.browser, 1200).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, xpath)))
    time.sleep(10)
    result_link = context.browser.find_element_by_xpath(find_link)
    assert "submitted" in result_link.text

@when('I click on the New Certification button')
def step_impl(context):
    new_cert_button = context.browser.find_element_by_xpath(xpath.new_cert_create)
    new_cert_button.click()

def product_option(product_name):
    product_dict
    { "Red Hat Enterprise Linux" : "1",
      "Red Hat Gluster Storage"  : "5",
      "Red Hat Enterprise Linux for Realtime" : "6",
      "Red Hat Enterprise Linux Server for ARM" : "7",
      "Red Hat Hypercoverged Infra" : "8",
      "Red Hat OpenStack Platform" : "3" }

    return product_dict[product_name]

@when('I select product "{product}"')
def step_impl(context, product):
    product_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('rh_product_id'))
    try:
        product_option_link.select_by_visible_text(context.config.userdata['product'])
    except:
        product_option_link.select_by_visible_text(product)

@when('I select RHOS version "{rhos_version}"')
def step_impl(context, rhos_version):
    rhos_version_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('rh_product_version_id'))
    rhos_version_option_link.select_by_visible_text(rhos_version)

@when('I select RHEL version "{rhel_version}" for "{cert_type}"')
def step_impl(context, rhel_version, cert_type):
    rhel_version_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('rh_product_version_id'))
    try:
         rhel_version_option_link.select_by_visible_text(context.config.userdata["rhel_version"])
    except:
         rhel_version_option_link.select_by_visible_text(rhel_version)

@when('I select RHEL version "{rhel_version}" for sandbox')
def step_impl(context, rhel_version):
    rhel_version_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('rh_product_version_id'))
    try:
        rhel_version_option_link.select_by_visible_text(context.config.userdata['rhel_version'])
    except:
        rhel_version_option_link.select_by_visible_text(rhel_version)    

@when('I select platform "{platform}"')
def step_impl(context, platform):
    platform_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('architecture'))
    try:
         platform_option_link.select_by_visible_text(context.config.userdata["platform"])
    except:
          platform_option_link.select_by_visible_text(platform)

@when('I select certification program "{program}" from Cloud or Hardware')
def step_impl(context, program):
    program_dropdown1 = context.browser.find_element_by_xpath('//*[@title="Hardware"]')
    program_dropdown1.click()
    programs = context.browser.find_elements_by_xpath("//span[@class='text']")
    for prog in programs:
        product_program = prog.text
        if product_program == program:
            prog.click()

@when('I select certification program "{program}" for Openstack')
def step_impl(context, program):
    webdriver.support.ui.WebDriverWait(context.browser, 1000)
    program_dropdown2 = context.browser.find_element_by_xpath('//*[@title="Openstack"]')
    program_dropdown2.click()
    programs1 = context.browser.find_elements_by_xpath("//span[@class='text']")
    for prog in programs1:
        product_program = prog.text
        if product_program == program:
            prog.click()

@when('I select RHEL version "{rhel_version}"')
def step_impl(context, rhel_version):
    rhel_version_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('rh_base_version_id'))
    rhel_version_option_link.select_by_visible_text(rhel_version)

@when('I select vendor "{vendorid}"')
def step_impl(context, vendorid):
    abbee_id_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('vendor_id'))
    try:
        abbee_id_option_link.select_by_visible_text(context.config.userdata['vendorid'])
    except:
        abbee_id_option_link.select_by_visible_text(vendorid)

@when('I select openstack product name "{product_name}"')
def step_impl(context, product_name):
    product_name_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('vendor_product_id'))
    product_name_option_link.select_by_visible_text(product_name)

@when('I select openstack plugin name "{plugin_name}"')
def step_impl(context, plugin_name):
    plugin_name_option_link = webdriver.support.ui.Select(context.browser.find_element_by_name('component'))
    plugin_name_option_link.select_by_visible_text(plugin_name)

@when('I select protocol "{protocol_name}" for manila')
def step_impl(context, protocol_name):
    protocol_list=['NFS','CIFS','HDFS','CephFS','MapRFS','GlusterFS']
    if protocol_name in protocol_list:
       element = context.browser.find_element_by_xpath('//*[@id="protocol-supported"][@value="%s"]' % protocol_name)
       element.click()
    elif protocol_name=='Select All':
       element = context.browser.find_element_by_xpath("//input[contains(@id,'checkboxSelector')]")
       element.click()
    elif protocol_name=='other':
       element = context.browser.find_element_by_xpath("//input[@id='other-protocol-name']")
       element.send_keys('other')

@when('I select protocol "{protocol_name}" for cinder')
def step_impl(context, protocol_name):
    protocol_list=['FCoE','FibreChannel','iSCSI','NFS','SAS','SATA','RoCE']
    if protocol_name in protocol_list:
       element = context.browser.find_element_by_xpath('//*[@id="protocol-supported"][@value="%s"]' % protocol_name)
       element.click()
    elif protocol_name=='Select All':
       element = context.browser.find_element_by_xpath("//input[contains(@id,'checkboxSelector')]")
       element.click()
    elif protocol_name=='other':
       element = context.browser.find_element_by_xpath("//input[@id='other-protocol-name']")
       element.send_keys('other')

@when('I click on create button')
def step_impl(context):
    context.browser.find_element_by_name("next-button").click()

@then('I see the "{cert_type}" certification created on rhcert')
def step_impl(context, cert_type):
    webdriver.support.ui.WebDriverWait(context.browser, 8200).until(webdriver.support.expected_conditions.visibility_of_element_located((webdriver.common.by.By.XPATH, xpath.cert_create_alert)))
    element = context.browser.find_element_by_xpath(xpath.cert_create_alert)
    success_message = element.text
    certification_id = success_message.split(' ')[10]
    context.data[context.data['parents'][cert_type]][cert_type]['cert_id_rhcert'] = certification_id

@when("I click on New Product")
def step_impl(context):
    new_product = context.browser.find_element_by_name("new-product")
    new_product.click()

@when('I select vendor "{vendor}" for product creation')
def step_impl(context, vendor):
    vendor_dropdown = context.browser.find_element_by_xpath(
            "/html/body/div[8]/form/div[2]/fieldset/div[1]/div/div/button")
   # vendor_dropdown = context.browser.find_element_by_xpath(
    #        "//button[@title='Abbee (ID: 1)']")
    vendor_dropdown.click()
    vendors = context.browser.find_elements_by_xpath("//span[@class='text']")
    try:
         vendor = context.config.userdata['vendor']
    except:
         vendor = vendor
    for ven in vendors:
        product_vendor = ven.text
        if product_vendor == vendor:
            ven.click()

@when('I select Program "{program}"')
def step_impl(context, program):
    program_dropdown = context.browser.find_element_by_xpath(
            "//button[@title='Red Hat Enterprise Linux Certified Hardware']")
    program_dropdown.click()
    programs = context.browser.find_elements_by_xpath("//span[@class='text']")
    for prog in programs:
        product_program = prog.text
        if product_program == program:
            prog.click()

@when('I enter sandbox name for "{cert_type}"')
def step_impl(context, cert_type):
    service_name = context.browser.find_element_by_name('new-sandbox-name')
    name = "%s_test-" % cert_type
    sandbox_name = name + time.strftime("%Y%m%d-%H%M%S")
    service_name.send_keys(sandbox_name)

@when('I enter sandbox name for Openstack')
def step_impl(context):
    service_name = context.browser.find_element_by_name('new-sandbox-name')
    name = "Openstack_test-"
    openstack_name = name + time.strftime("%Y%m%d-%H%M%S")
    service_name.send_keys(openstack_name)

@step('I should see the Sandbox certification created for "{cert_type1}"')
def step_impl(context, cert_type1):
    element = context.browser.find_element_by_xpath("//div[@class='toast-pf alert alert-info alert-dismissible']")
    success_message = element.text
    certification_id1 = success_message.split()[-1]
    context.data[context.data['parents'][cert_type1]][cert_type1]['cert_id_rhcert'] = certification_id1

@when('I input Cloud/Service Name')
def step_impl(context):
    service_name = context.browser.find_element_by_xpath(
        "//input[@name='product_name'][@id='product_name']")
    name = "cloud_test-"
    cloud_name = name + time.strftime("%Y%m%d-%H%M%S")
    service_name.send_keys(cloud_name)

@when('I input Public Catalog URL "{URL}"')
def step_impl(context, URL):
    catalog_url = context.browser.find_element_by_xpath(
        "//input[@name='product_url'][@id='product_url']")
    catalog_url.send_keys(URL)

@when('I select Supported Regions')
def step_impl(context):
    supported_region1 = context.browser.find_element_by_xpath(
        "//input[@name='supported_region'][@value='EMEA']")
    supported_region1.click()

@when('I input Supported Languages "{Language}"')
def step_impl(context, Language):
    supported_languages = context.browser.find_element_by_name("supported_languages")
    supported_languages.send_keys(Language)

@when('I input 3rd Party Certification "{Certification}"')
def step_impl(context, Certification):
    certification = context.browser.find_element_by_name("third_party_certifications")
    certification.send_keys(Certification)

@then('I see the "{cert_type}" product created on rhcert')
def step_impl(context, cert_type):
    webdriver.support.ui.WebDriverWait(context.browser, 360).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, "/html/body/div[5]/div/div[contains(.,"
                                                    "'Created new Product for Program')]")))

    element = context.browser.find_element_by_xpath("/html/body/div[5]/div/div[contains(.,"
                                                    "'Created new Product for Program')]")

    success_message = element.text
    product_id = success_message.split(' ')[12]
    context.data[context.data['parents'][cert_type]][cert_type]['product_id_rhcert'] = product_id

@then('I see the "{cert_type}" certification created on rhcert against new Product')
def step_impl(context, cert_type):
    webdriver.support.ui.WebDriverWait(context.browser, 360).until(
        webdriver.support.expected_conditions.visibility_of_element_located((
            webdriver.common.by.By.XPATH, "/html/body/div[6]/div/div[contains(.,"
                                                    "'Created Certification for Program')]")))

    element = context.browser.find_element_by_xpath("/html/body/div[6]/div/div[contains(.,"
                                                    "'Created Certification for Program')]")
    success_message = element.text
    certification_id = success_message.split(' ')[10]
    context.data[context.data['parents'][cert_type]][cert_type]['cert_id_rhcert'] = certification_id

@when('I select type of plugin "{plugin_type}"')
def step_impl(context, plugin_type):
     context.browser.find_element_by_id('networking_plugin_type_' + plugin_type).click()

@step('I click on Details button')
def step_impl(context):
    detail = context.browser.find_element_by_xpath("//a[contains(.,'Details')]")
    detail.click()

@then('I save the cert_nid of "{cert_type}" created on rhcert')
def step_impl(context, cert_type):
    nid_path = context.browser.find_element_by_xpath("id('DataTables_Table_0')/tbody[2]/tr[4]/td[2]")
    cert_nid = nid_path.text
    context.data[context.data['parents'][cert_type]][cert_type]['cert_nid_rhcert'] = cert_nid
