try:
    import datetime
    import time
    from behave import when, then
    from rhcert import get_node_id, search

    import utilities
    utilities.safe_selenium_imports(__file__, globals())
except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.webdriver.support.ui import Select

@when('I click on certification create next button')
def step_impl(context):
    context.browser.find_element_by_name("adding").click()


@when('I create certification from rhcert for cloud plugin')
def step_impl(context):
    context.browser.find_element_by_name(
        "image_name").send_keys("cloud_cert_rhcert")
    context.browser.find_element_by_name("next").click()


@then('I could see "{cert_type}" certification created on rhcert')
def step_impl(context, cert_type):
    success_message = context.browser.find_element_by_css_selector(
        "label.col-md-4.control-label").text
    certification_id = success_message.split(' ')[3]
    context.data[context.data['parents'][cert_type]][
        cert_type]['cert_id_rhcert'] = certification_id
    context.browser.find_element_by_name("done").click()


@when('I click on Certification')
def step_impl(context):
    context.browser.find_element_by_name("adding").click()


@when('I have "{cert_type}" product available')
def step_impl(context, cert_type):
    node_id = get_node_id(context, cert_type)
    search(context, node_id)
    product_name = context.browser.find_element_by_xpath(
        "//table/tbody/tr/td/a").text
    product_name_link = context.browser.find_element_by_link_text(product_name)
    product_name_link.click()


def common_steps(context, version):
    cert_name = version
    suffix = datetime.datetime.now().strftime("%y%m%d_%H%M%S")
    cert_name = "_".join([cert_name, suffix])
    cert_version = context.browser.find_element_by_name(
        'vendor_product_version')
    cert_version.send_keys(cert_name)
    webdriver.support.ui.Select(context.browser.find_element_by_id("program_version")
           ).select_by_visible_text("Red Hat OpenStack Platform " + version)
    hostname = context.browser.find_element_by_name('hostname')
    hostname.send_keys(version + '.rhcert')
    source_url = context.browser.find_element_by_name('source_url')
    source_url.send_keys('http://www.example.com/source_url')
    blueprint_url = context.browser.find_element_by_name('blueprint_url')
    blueprint_url.send_keys('http://www.example.com/blueprint_url')
    install_guide = context.browser.find_element_by_name('install_guide')
    install_guide.send_keys('http://www.example.com/install_guide')


def api_implemented_xpath(context, plugin_type):
    if plugin_type == "cinder":
        api_list = list(range(2, 8))
    if plugin_type == "neutron":
        api_list = list(range(18, 37))
    if plugin_type == "sahara":
        api_list = list(range(8, 17))
    for i in api_list:
        context.browser.find_element_by_xpath(
            "(//input[@name='api_implemented'])[" + str(i) + "]").click()


def select_suppported_services(context, supported_service):
    for services in supported_service:
        context.browser.find_element_by_name(
            "supported_service_" + services).click()


@when('I create RHOS "{version}" certification from rhcert for cinder plugin')
def step_impl(context, version):
    supported_service = [
        "volume_create", "snapshot_to_volume",
        "storage_protocol", "volume_attach",
        "volume_stat", "clone_volume",
        "free_capacity_gb", "total_capacity_gb",
        "snapshot_create", "image_to_volume", "extend_volume",
        "reserved_percentage", "vendor_name"]
    common_steps(context, version)
    context.browser.find_element_by_name("api_implemented").click()
    api_implemented_xpath(context, "cinder")
    context.browser.find_element_by_name("next").click()
    select_suppported_services(context, supported_service)
    context.browser.find_element_by_name("supported_protocol").clear()
    context.browser.find_element_by_name(
        "supported_protocol").send_keys("iscsi")
    context.browser.find_element_by_css_selector(
        "button[name=\"next\"]").click()


@when('I create RHOS "{version}" certification from rhcert for neutron plugin')
def step_impl(context, version):
    common_steps(context, version)
    webdriver.support.ui.Select(context.browser.find_element_by_id("component")
           ).select_by_visible_text("certification-networking-plugin")
    api_implemented_xpath(context, "neutron")
    context.browser.find_element_by_name("next").click()
    context.browser.find_element_by_id("networking_plugin_type_ml2").click()
    context.browser.find_element_by_name(
        "isolation_technology").send_keys("test")
    context.browser.find_element_by_name(
        "included_component").send_keys("test")
    context.browser.find_element_by_name("ovs_reference").send_keys("ovs")
    context.browser.find_element_by_name("ovs_userspace").send_keys("test")
    context.browser.find_element_by_name("ovs_kernel").send_keys("test")
    context.browser.find_element_by_id("support_lbaas_yes").click()
    context.browser.find_element_by_id("support_vpnaas_yes").click()
    context.browser.find_element_by_id("support_fwaas_yes").click()
    context.browser.find_element_by_name("solution_desc").send_keys("test")
    context.browser.find_element_by_name(
        "solution_dependancy").send_keys("test")
    context.browser.find_element_by_id("is_ha_yes").click()
    context.browser.find_element_by_name("ha_deployment").send_keys("test")
    context.browser.find_element_by_id("hw_reference_conf_arista").click()
    context.browser.find_element_by_css_selector(
        "button[name=\"next\"]").click()


@when(
    'I create RHOS "{version}" certification from rhcert for other_application plugin')
def step_impl(context, version):
    common_steps(context, version)
    webdriver.support.ui.Select(context.browser.find_element_by_id("component")
           ).select_by_visible_text("certification-other-application")
    context.browser.find_element_by_name("next").click()


@when('I create RHOS "{version}" certification from rhcert for sahara plugin')
def step_impl(context, version):
    common_steps(context, version)
    webdriver.support.ui.Select(context.browser.find_element_by_id("component")
           ).select_by_visible_text("certification-dataprocessing-plugin")
    api_implemented_xpath(context, "sahara")
    context.browser.find_element_by_name("next").click()
    context.browser.find_element_by_name(
        "supported_provisioning_plugin_mapr").click()
    context.browser.find_element_by_name(
        "supported_provisioning_plugin_hdp").click()
    context.browser.find_element_by_name(
        "supported_provisioning_plugin_cloudera").click()
    context.browser.find_element_by_name("node_group").send_keys("test")
    context.browser.find_element_by_name("re_version").send_keys("test")
    context.browser.find_element_by_name(
        "security_group_api").send_keys("test")
    context.browser.find_element_by_css_selector(
        "button[name=\"next\"]").click()


@when('I create RHOS "{version}" certification from rhcert for management_app plugin')
def step_impl(context, version):
    common_steps(context, version)
    webdriver.support.ui.Select(context.browser.find_element_by_id("component")
           ).select_by_visible_text("certification-management-application")
    context.browser.find_element_by_name("next").click()


@when('I create RHOS "{version}" certification from rhcert for manila plugin')
def step_impl(context, version):
    common_steps(context, version)
    webdriver.support.ui.Select(context.browser.find_element_by_id("component")
           ).select_by_visible_text("certification-filestorage-plugin")
    # context.browser.find_element_by_name("api_implemented").click()
    # api_implemented_xpath(context, "cinder")
    # context.browser.find_element_by_name("next").click()
    # select_suppported_services(context, supported_service)
    # context.browser.find_element_by_name("supported_protocol").clear()
    # context.browser.find_element_by_name(
    #    "supported_protocol").send_keys("iscsi")
    context.browser.find_element_by_css_selector(
        "button[name=\"next\"]").click()
    time.sleep(5)
    context.browser.find_element_by_css_selector(
        "button[name=\"next\"]").click()
