try:
    import os
    import re
    from random import choice
    from string import ascii_uppercase
    from xml.dom import minidom

    import paramiko
    from behave import given, when, then

    import utilities
    utilities.safe_selenium_imports(__file__, globals())

except ImportError as e:
    print("ImportError in {0}: {1}".format(__file__, e))
    raise e

# from selenium.webdriver.common.by import By
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.webdriver.support.ui import WebDriverWait, Select


def ssh_to_host(host, username, password):
    """
    SSH into host
    """
    client = paramiko.SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    try:
        client.connect(host, username=username, key_filename=None,
                       look_for_keys=False, password=password)
        return client
    except Exception as e:
        assert '*** Failed to connect to %s %s' % (host, e)
        return "FAILED"


@given("I have sshed into RHCERT")
def step_impl(context):
    host = context.data['common']['rhcert']['ip']
    username = context.data['common']['rhcert']['ssh_username']
    password = context.data['common']['rhcert']['ssh_password']
    context.rhcert_ssh = ssh_to_host(host, username, password)


@then("I close the RHCERT ssh connection")
def step_impl(context):
    context.rhcert_ssh.close()


@given("I have sshed into Cloud SUT")
def step_impl(context):
    host = context.data['platform']['cloud']['sut']['host']
    username = context.data['platform']['cloud']['sut']['ssh_username']
    password = context.data['platform']['cloud']['sut']['ssh_password']
    context.cloud_ssh = ssh_to_host(host, username, password)


@then("I close the SUT ssh connection")
def step_impl(context):
    context.cloud_ssh.close()


@given("'{package}' package is installed on '{machine}'")
def step_impl(context, package, machine):
    cmd = "rpm -qi %s | wc -l" % package
    if machine == 'rhcert':
        stdin, stdout, stderr = context.rhcert_ssh.exec_command(cmd)
    else:
        stdin, stdout, stderr = context.cloud_ssh.exec_command(cmd)
    assert (int(stdout.read()) > 1) is True


# service httpd status | grep running
@when('"{service}" service is "{status}" on "{machine}"')
def step_impl(context, service, status, machine):
    if service == "httpd":
        cmd = "service %s status | grep %s | wc -l" % (service, status)
    else:
        service = "openstack-%s-api" % service
        cmd = "service %s status | grep %s | wc -l" % (service, status)
    if machine == 'rhcert':
        stdin, stdout, stderr = context.rhcert_ssh.exec_command(cmd)
    else:
        stdin, stdout, stderr = context.cloud_ssh.exec_command(cmd)
    assert (len(stdout.readlines()) > 0) is True
    if machine == 'rhcert':
        cmd = "systemctl stop firewalld"
        stdin, stdout, stderr = context.rhcert_ssh.exec_command(cmd)
 

# rhcertd stop | grep 'not running'


@when("I stop the rhcertd service")
def step_impl(context):
    context.rhcert_ssh.exec_command("rhcertd stop")
    stdin, stdout, stderr = context.rhcert_ssh.exec_command(
        "rhcertd status | wc -l")
    assert (len(stdout.readlines()) > 0) is True


@when("I start the rhcertd service")
def step_impl(context):
    context.rhcert_ssh.exec_command("rhcertd start")
    stdin, stdout, stderr = context.rhcert_ssh.exec_command(
        "rhcertd status | wc -l")
    assert (len(stdout.readlines()) > 0) is True


@when("I clean the RHCERT system")
def step_impl(context):
    stdin, stdout, stderr = context.rhcert_ssh.exec_command(
        "rhcert-cli clean all")
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I clean the cloud SUT")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-cli clean all")
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I clean the cloud SUT for rhcert-ci")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-ci clean all")
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I plan test on cloud SUT")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-cli plan")
    stdin.write("no")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I plan test on cloud SUT for rhcert-ci")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-ci plan")
    stdin.write("no")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I print the test plan on cloud SUT")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-cli print")
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I print the test plan on cloud SUT for rhcert-ci")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-ci print")
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I register SUT with LTS")
def step_impl(context):
    lts = context.data['common']['rhcert']['ip']
    cmd = "rhcert-cli register --server-ip %s" % lts
    stdin, stdout, stderr = context.cloud_ssh.exec_command(cmd)
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    stdin.write("yes")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I save a copy of current test results")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-cli save")
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I save a copy of current test results for rhcert-ci")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-ci save")
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I run all tests on cloud SUT")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-cli run")
    stdin.write("no")
    stdin.write('\n')
    stdin.flush()
    stdin.write("Automate")
    stdin.write('\n')
    stdin.flush()
    stdin.write("Automate")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I run all tests on cloud SUT for rhcert-ci")
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-ci run")
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)

@when("I run '{test_type}' test on cloud SUT")
def step_impl(context, test_type):
    cmd = "rhcert-cli run --test cloud/%s" % test_type
    stdin, stdout, stderr = context.cloud_ssh.exec_command(cmd)
    stdin.write("no")
    stdin.write('\n')
    stdin.flush()
    stdin.write("Automate")
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I submit '{cert_type}' certification results to Red Hat")
def step_impl(context, cert_type):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
        "rhcert-cli submit")
    username = context.data['common']['rhcert']['sso_username']
    password = context.data['common']['rhcert']['sso_password']
    cert_id = context.data[context.data['parents']
                           [cert_type]][cert_type]['cert_id_rhcert']
    stdin.write(username)
    stdin.write('\n')
    stdin.flush()
    stdin.write(password)
    stdin.write('\n')
    stdin.flush()
    stdin.write('No')
    stdin.write('\n')
    stdin.flush()
    stdin.write(cert_id)
    stdin.write('\n')
    stdin.flush()
    stdin.write('no')
    stdin.write('\n')
    stdin.flush()
    outlines = stdout.readlines()
    resp = ''.join(outlines)
    print(resp)


@when("I configure the machine to '{config}'")
def step_impl(context,config):
    stdin, stdout, stderr = context.rhcert_ssh.exec_command(
        "rhcert-util configure %s" % config)
    assert (len(stdout.readlines()) > 0) is True

@when("I verify machine is configured to '{config}'")
def step_impl(context,config):
    stdin, stdout, stderr = context.rhcert_ssh.exec_command(
        "rhcert-util configure %s | grep 'Current Server configuration: %s'" % (config, config))

@when('I change phase to invalid')
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
            "cat /etc/rhcert.xml")
    outlines = stdout.read()
    xmldoc = minidom.parseString(outlines)
    tagname = xmldoc.getElementsByTagName("urls")
    context.data['phase'] = tagname[0].attributes['phase'].value
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
            "sed -i \"s/<urls phase='"+context.data['phase']+"'>/<urls phase='qa1'>/g\" /etc/rhcert.xml")

@when('I check status of security test on cloud SUT')
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
            "rhcert-cli print")
    f1 = stdout.readlines()[-2].split()[2]
    assert f1 == "REVIEW"

@when('I change phase back to valid')
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
            "sed -i -e \"s/<urls phase='qa1'>/<urls phase='"+context.data['phase']+"'>/g\" /etc/rhcert.xml")

@when('I assert security test status')
def step_impl(context):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
            "rhcert-cli print")
    f1 = stdout.readlines()[-2].split()[2]
    assert f1 == "PASS"

@when('I save a copy of current test results for "{cert_type}"')
def step_impl(context, cert_type):
    stdin, stdout, stderr = context.cloud_ssh.exec_command(
            "rhcert-cli save")
    output = stdout.readlines()[-2].split()
    a1 = output[0]
    a2 = output[1]
    save_results_path = a1 + "\ " + a2
    context.data[context.data['parents'][cert_type]][cert_type]['save_results_path'] = save_results_path

@then('I update the test results for "{cert_type}"')
def step_impl(context, cert_type):
    result_path = context.data[context.data['parents'][cert_type]][cert_type]['save_results_path']
    cmd = "rhcert-util update --results {0}".format(result_path)
    stdin, stdout, stderr = context.cloud_ssh.exec_command(cmd)

def get_save_results_path(context):
    results_path = context.data[context.data['parents'][cert_type]][cert_type]['save_results_path']
    return results_path

